package com.yiidian.springboot.demo;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yiidian.springboot.demo.controller.ArticleController;
import com.yiidian.springboot.demo.pojo.Article;
import com.yiidian.springboot.demo.service.ArticleService;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpMethod;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
/**
 * 一点教程网：http://www.yiidian.com
 */
@Slf4j
@AutoConfigureMockMvc  //由Spring创建MockMvc对象
@WebMvcTest  //只需要测试Controller层，不需要真正测试Service，用这个注解就够了！
public class ArticleRestControllerTest {

    //Mock对象
    @Autowired //我们可以直接注入使用
    private MockMvc mockMvc;

    @MockBean // 模拟创建一个ArticleService对象
    private ArticleService articleService;

    //测试方法
    @Test
    public void saveArticle() throws Exception {

        String article = "{\n" +
                "    \"id\": 1,\n" +
                "    \"categoryName\": \"Java\",\n" +
                "    \"title\": \"Spring Boot入门到精通\",\n" +
                "    \"content\": \"c\",\n" +
                "    \"createTime\": \"2022-04-21 17:26:30\",\n" +
                "    \"authors\":[{\"name\":\"zhangsan\",\"age\":18},{\"name\":\"lisi\",\"age\":37}]\n" +
                "}";

        ObjectMapper objectMapper = new ObjectMapper();
        Article articleObj = objectMapper.readValue(article,Article.class);

        //打桩
        Mockito.when(articleService.saveArticle(articleObj)).thenReturn("ok");

        //模拟http请求
        MvcResult result = mockMvc.perform(
                MockMvcRequestBuilders.request(HttpMethod.POST, "/rest/articles")
                        .contentType("application/json").content(article))
                .andExpect(MockMvcResultMatchers.jsonPath("$.data").value("ok"))
                .andReturn();

        result.getResponse().setCharacterEncoding("UTF-8");
        log.info(result.getResponse().getContentAsString());

    }
}
